@@ -175,33 +175,33 @@ static const struct mx6_ddr_sysinfo skov_imx6_sysinfo_4x128Mb_1066MHz = {
 	.dsize = 2, /* 64 bit wide = 4 devices, 16 bit each */
 	.cs_density = 8, /* four 2 GBit devices connected */
 	.ncs = 1, /* one CS line for all devices */
-	.cs1_mirror = 1, /* TODO */
-	.bi_on = 1, /* TODO */
+	.cs1_mirror = 1,
+	.bi_on = 1,
 	.rtt_nom = 1, /* MX6_MMDC_P0_MPODTCTRL -> 0x00022227 */
 	.rtt_wr = 0, /* is LW_EN is 0 in their code */
-	.ralat = 5, /* TODO */
-	.walat = 1, /* TODO */
-	.mif3_mode = 3, /* TODO */
-	.rst_to_cke = 0x23, /* used in their code as well */
-	.sde_to_rst = 0x10, /* used in their code as well */
-	.pd_fast_exit = 0, /* TODO */
+	.ralat = 5,
+	.walat = 0,
+	.mif3_mode = 3,
+	.rst_to_cke = 0x23,
+	.sde_to_rst = 0x10,
+	.pd_fast_exit = 1,
 };
 
 /* calibration info for the "max performance" and "high performance" */
 static const struct mx6_mmdc_calibration skov_imx6_calib_4x128Mb_1066MHz = {
-	.p0_mpwldectrl0 = 0x0011000E,
-	.p0_mpwldectrl1 = 0x000E001B,
-	.p0_mpdgctrl0 = 0x42720306,
-	.p0_mpdgctrl1 = 0x026F0266,
-	.p0_mprddlctl = 0x45393B3E,
-	.p0_mpwrdlctl = 0x40434541,
+	.p0_mpwldectrl0 = 0x00230023,
+	.p0_mpwldectrl1 = 0x0029001E,
+	.p0_mpdgctrl0 = 0x43400350,
+	.p0_mpdgctrl1 = 0x03380330,
+	.p0_mprddlctl = 0x3E323638,
+	.p0_mpwrdlctl = 0x383A3E3A,
 
-	.p1_mpwldectrl0 = 0x00190015,
-	.p1_mpwldectrl1 = 0x00070018,
-	.p1_mpdgctrl0 = 0x4273030A,
-	.p1_mpdgctrl1 = 0x02740240,
-	.p1_mprddlctl = 0x403A3747,
-	.p1_mpwrdlctl = 0x473E4A3B,
+	.p1_mpwldectrl0 = 0x001F002A,
+	.p1_mpwldectrl1 = 0x001A0028,
+	.p1_mpdgctrl0 = 0x43300340,
+	.p1_mpdgctrl1 = 0x03340300,
+	.p1_mprddlctl = 0x383A3242,
+	.p1_mpwrdlctl = 0x4232463A,
 };
 
 /* ------------------------------------------------------------------------ */
@@ -215,21 +215,21 @@ static struct mx6dq_iomux_ddr_regs ddr_iomux_q = {
 	.dram_sdqs5 = 0x00000030,
 	.dram_sdqs6 = 0x00000030,
 	.dram_sdqs7 = 0x00000030,
-	.dram_dqm0 = 0x00020030,
-	.dram_dqm1 = 0x00020030,
-	.dram_dqm2 = 0x00020030,
-	.dram_dqm3 = 0x00020030,
-	.dram_dqm4 = 0x00020030,
-	.dram_dqm5 = 0x00020030,
-	.dram_dqm6 = 0x00020030,
-	.dram_dqm7 = 0x00020030,
-	.dram_cas = 0x00020030,
-	.dram_ras = 0x00020030,
-	.dram_sdclk_0 = 0x00020030,
-	.dram_sdclk_1 = 0x00020030,
+	.dram_dqm0 = 0x00000030,
+	.dram_dqm1 = 0x00000030,
+	.dram_dqm2 = 0x00000030,
+	.dram_dqm3 = 0x00000030,
+	.dram_dqm4 = 0x00000030,
+	.dram_dqm5 = 0x00000030,
+	.dram_dqm6 = 0x00000030,
+	.dram_dqm7 = 0x00000030,
+	.dram_cas = 0x00000030,
+	.dram_ras = 0x00000030,
+	.dram_sdclk_0 = 0x00000030,
+	.dram_sdclk_1 = 0x00000030,
 	.dram_sdcke0 = 0x00003000,
 	.dram_sdcke1 = 0x00003000,
-	.dram_reset = 0x00020030,
+	.dram_reset = 0x00000030,
 	.dram_sdba2 = 0x00000000,
 	.dram_sdodt0 = 0x00003030,
 	.dram_sdodt1 = 0x00003030,
@@ -296,25 +296,25 @@ static const struct mx6_ddr_sysinfo skov_imx6_sysinfo_2x128Mb_800MHz = {
 	.dsize = 1, /* 32 bit wide = 2 devices, 16 bit each */
 	.cs_density = 4, /* two 2 GBit devices connected */
 	.ncs = 1, /* one CS line for all devices */
-	.cs1_mirror = 1, /* TODO */
-	.bi_on = 1, /* TODO */
+	.cs1_mirror = 1,
+	.bi_on = 1,
 	.rtt_nom = 1, /* MX6_MMDC_P0_MPODTCTRL -> 0x00022227 */
 	.rtt_wr = 0, /* is LW_EN is 0 in their code */
-	.ralat = 5, /* TODO */
-	.walat = 1, /* TODO */
-	.mif3_mode = 3, /* TODO */
-	.rst_to_cke = 0x23, /* used in their code as well */
-	.sde_to_rst = 0x10, /* used in their code as well */
-	.pd_fast_exit = 0, /* TODO */
+	.ralat = 5,
+	.walat = 0,
+	.mif3_mode = 3,
+	.rst_to_cke = 0x23,
+	.sde_to_rst = 0x10,
+	.pd_fast_exit = 1,
 };
 
 static const struct mx6_mmdc_calibration skov_imx6_calib_2x128Mb_800MHz = {
-	.p0_mpwldectrl0 = 0x0040003C,
-	.p0_mpwldectrl1 = 0x0032003E,
-	.p0_mpdgctrl0 = 0x42350231,
-	.p0_mpdgctrl1 = 0x021A0218,
-	.p0_mprddlctl = 0x4B4B4E49,
-	.p0_mpwrdlctl = 0x3F3F3035,
+	.p0_mpwldectrl0 = 0x004A004B,
+	.p0_mpwldectrl1 = 0x00420046,
+	.p0_mpdgctrl0 = 0x42400240,
+	.p0_mpdgctrl1 = 0x02300230,
+	.p0_mprddlctl = 0x464A4A4A,
+	.p0_mpwrdlctl = 0x32342A32,
 };
 
 /* ------------------------------------------------------------------------ */
@@ -328,21 +328,21 @@ static const struct mx6sdl_iomux_ddr_regs ddr_iomux_s = {
 	.dram_sdqs5 = 0x00000030,
 	.dram_sdqs6 = 0x00000030,
 	.dram_sdqs7 = 0x00000030,
-	.dram_dqm0 = 0x00020030,
-	.dram_dqm1 = 0x00020030,
-	.dram_dqm2 = 0x00020030,
-	.dram_dqm3 = 0x00020030,
-	.dram_dqm4 = 0x00020030,
-	.dram_dqm5 = 0x00020030,
-	.dram_dqm6 = 0x00020030,
-	.dram_dqm7 = 0x00020030,
-	.dram_cas = 0x00020030,
-	.dram_ras = 0x00020030,
-	.dram_sdclk_0 = 0x00020030,
-	.dram_sdclk_1 = 0x00020030,
+	.dram_dqm0 = 0x00000030,
+	.dram_dqm1 = 0x00000030,
+	.dram_dqm2 = 0x00000030,
+	.dram_dqm3 = 0x00000030,
+	.dram_dqm4 = 0x00000030,
+	.dram_dqm5 = 0x00000030,
+	.dram_dqm6 = 0x00000030,
+	.dram_dqm7 = 0x00000030,
+	.dram_cas = 0x00000030,
+	.dram_ras = 0x00000030,
+	.dram_sdclk_0 = 0x00000030,
+	.dram_sdclk_1 = 0x0000030,
 	.dram_sdcke0 = 0x00003000,
 	.dram_sdcke1 = 0x00003000,
-	.dram_reset = 0x00020030,
+	.dram_reset = 0x00000030,
 	.dram_sdba2 = 0x00000000,
 	.dram_sdodt0 = 0x00003030,
 	.dram_sdodt1 = 0x00003030,
