@@ -11,7 +11,7 @@
 
 static int regmap_spi_write(void *context, const void *data, size_t count)
 {
-	struct device_d *dev = context;
+	struct device *dev = context;
 	struct spi_device *spi = to_spi_device(dev);
 
 	return spi_write(spi, data, count);
@@ -21,7 +21,7 @@ static int regmap_spi_read(void *context,
 			   const void *reg, size_t reg_size,
 			   void *val, size_t val_size)
 {
-	struct device_d *dev = context;
+	struct device *dev = context;
 	struct spi_device *spi = to_spi_device(dev);
 
 	return spi_write_then_read(spi, reg, reg_size, val, val_size);
